' hid_desc.bas
' USB descriptors for a HID device
' 9/16/08 microEngineering Labs, Inc.

'USBMEMORYADDRESS Con	$400	' USB RAM starts here (set in device header file)
USBMEMORYSIZE	Con	256	' USB RAM size in bytes
USBReservedMemory Var Byte[USBMEMORYSIZE] USBMEMORYADDRESS	' Reserve memory used by USB assembler code

	Goto	hid_desc_end1	' Skip over all of the USB assembler code

	Asm

#define USB_EP0_BUFF_SIZE	64	; 8, 16, 32, or 64
#define USB_MAX_NUM_INT		1
#define USB_MAX_EP_NUMBER	1
#define NUM_CONFIGURATIONS	1
#define NUM_INTERFACES		1

#define UCFG_VAL	USB_PULLUP_ENABLE|USB_INTERNAL_TRANSCEIVER|USB_FULL_SPEED|USB_PING_PONG__NO_PING_PONG
;#define UCFG_VAL	USB_PULLUP_ENABLE|USB_INTERNAL_TRANSCEIVER|USB_LOW_SPEED|USB_PING_PONG__NO_PING_PONG

;#define USE_SELF_POWER_SENSE_IO
;#define USE_USB_BUS_SENSE_IO

#define USB_POLLING

; HID
; Endpoints Allocation
#define HID_INTF_ID		       0x00
#define HID_EP			       1
#define HID_INT_OUT_EP_SIZE	   1
#define HID_INT_IN_EP_SIZE	   1
#define HID_NUM_OF_DSC		   1

	include	"usb_hid.asm"	; Include rest of USB files, starting with HID class code

; ******************************************************************
; This table is polled by the host immediately after USB Reset has been released.
; This table defines the maximum packet size EP0 can take.
; See section 9.6.1 of the Rev 1.0 USB specification.
; These fields are application DEPENDENT. Modify these to meet
; your specifications.
; ******************************************************************
DeviceDescriptor
	retlw	(EndDeviceDescriptor-DeviceDescriptor)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_DEVICE	; DEVICE descriptor type
	retlw	0x10		; USB Spec Release Number in BCD format - 1.10 (low byte)
	retlw	0x01		; (high byte)
	retlw	0x00		; Class Code
	retlw	0x00		; Subclass code
	retlw	0x00		; Protocol code
	retlw	USB_EP0_BUFF_SIZE	; Max packet size for EP0
	retlw	0x07		; Vendor ID - 0x2007 is SCUMARI Vendor ID (low byte)
	retlw	0x20		; (high byte)
	retlw	0x47		; Product ID: Joystick FY (low byte)
	retlw	0x12		; (high byte)
	retlw	0x01		; Device release number in BCD format (low byte)
	retlw	0x00		; (high byte)
	retlw	0x01		; Manufacturer string index
	retlw	0x02		; Product string index
	retlw	0x00		; Device serial number string index
	retlw	NUM_CONFIGURATIONS	; Number of possible configurations
EndDeviceDescriptor

; ******************************************************************
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application DEPENDENT. 
; Modify these to meet your specifications.
; ******************************************************************
; Configuration pointer table
USB_CD_Ptr
Configs
	db	low Config1, high Config1
	db	upper Config1, 0

; Configuration Descriptor
Config1
	retlw	(Interface1-Config1)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_CONFIGURATION	; CONFIGURATION descriptor type
Config1Len
	retlw	low ((EndConfig1-Config1)/2)	; Total length of data for this cfg
	retlw	high ((EndConfig1-Config1)/2)
	retlw	NUM_INTERFACES	; Number of interfaces in this cfg
	retlw	0x01		; Index value of this configuration
	retlw	0x00		; Configuration string index
	retlw	_DEFAULT|_SELF	; Attributes
	retlw	50		; Max power consumption (2X mA)
Interface1
	retlw	(HIDDescriptor1-Interface1)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_INTERFACE	; INTERFACE descriptor type
	retlw	0x00		; Interface Number
	retlw	0x00		; Alternate Setting Number
	retlw	0x01		; Number of endpoints in this intf
	retlw	HID_INTF	; Class code
	retlw	BOOT_INTF_SUBCLASS	; Subclass code
	retlw	HID_PROTOCOL_MOUSE	; Protocol code
	retlw	0x00		; Interface string index
HIDDescriptor1
	retlw	(Endpoint1-HIDDescriptor1)/2	; Size of this descriptor in bytes
	retlw	DSC_HID		; HID descriptor type
	retlw	0x11		; HID Spec Release Number in BCD format - 1.11 (low byte)
	retlw	0x01		; (high byte)
	retlw	0x00		; Country Code (0x00 for Not supported)
	retlw	HID_NUM_OF_DSC	; Number of class descriptors
	retlw	DSC_RPT		; Report descriptor type (HID)
ReportDescriptor1Len
	retlw	low ((EndReportDescriptor1-ReportDescriptor1)/2)
	retlw	high ((EndReportDescriptor1-ReportDescriptor1)/2)
Endpoint1
	retlw	(EndConfig1-Endpoint1)/2	; Size of this descriptor in bytes
	retlw	USB_DESCRIPTOR_ENDPOINT	; ENDPOINT descriptor type
	retlw	HID_EP|_EP_IN	; Endpoint Address
	retlw	_INT		; Attributes
	retlw	low (HID_INT_IN_EP_SIZE)	; Size of the endpoint buffer in bytes (low byte)
	retlw	high (HID_INT_IN_EP_SIZE)	; (high byte)
	retlw	0x0A		; Polling interval (10ms)
EndConfig1

ReportDescriptor1
    RETLW 0x05
    RETLW 0x01                    ; USAGE_PAGE (Generic Desktop)
    RETLW 0x09
    RETLW 0x04                    ; USAGE (Joystick)
    RETLW 0xa1
    RETLW 0x01                    ; COLLECTION (Application)
                         ; ------------ Hat Switches -------------------
    RETLW 0x09
    RETLW 0x39                    ;   USAGE (Hat switch)
    RETLW 0x15
    RETLW 0x00                    ;   LOGICAL_MINIMUM (0)
    RETLW 0x25
    RETLW 0x03                    ;   LOGICAL_MAXIMUM (3)
    RETLW 0x35
    RETLW 0x00                    ;   PHYSICAL_MINIMUM (0)
    RETLW 0x46
    RETLW 0x0e
    RETLW 0x01                    ;   PHYSICAL_MAXIMUM (270)
    RETLW 0x65
    RETLW 0x14                    ;   UNIT (Eng Rot:Angular Pos)
    RETLW 0x75
    RETLW 0x04                    ;   REPORT_SIZE (4)
    RETLW 0x95
    RETLW 0x01                    ;   REPORT_COUNT (1)
    RETLW 0x81              
    RETLW 0x02                    ;   INPUT (Data, Variable, Absolute)
                         ; ------------ Buttons ------------------------
    RETLW 0x05 
    RETLW 0x09                    ;   USAGE_PAGE (Button)
    RETLW 0x19 
    RETLW 0x01                    ;   USAGE_MINIMUM (Button 1)
    RETLW 0x29 
    RETLW 0x04                    ;   USAGE_MAXIMUM (Button 4)
    RETLW 0x15 
    RETLW 0x00                    ;   LOGICAL_MINIMUM (0)
    RETLW 0x25 
    RETLW 0x01                    ;   LOGICAL_MAXIMUM (1)
    ;RETLW 0x35
    ;RETLW 0x00                    ;   PHYSICAL_MINIMUM (0)
    RETLW 0x45
    RETLW 0x00                    ;   PHYSICAL_MAXIMUM (0)
    RETLW 0x75 
    RETLW 0x01                    ;   REPORT_SIZE (1)
    RETLW 0x95 
    RETLW 0x04                    ;   REPORT_COUNT (4)
    RETLW 0x55
    RETLW 0x00                    ;   UNIT_EXPONENT (0)
    RETLW 0x65
    RETLW 0x00                    ;   UNIT (None)
    RETLW 0x81 
    RETLW 0x02                    ;   INPUT (Data Var Abs)
    RETLW 0xc0                    ; END_COLLECTION
EndReportDescriptor1    
 
    endasm
hid_desc_end1			' End of skipped over USB assembler code
	Goto	hid_desc_end	' Skip over all of the USB assembler code

	Asm

; String pointer table
USB_SD_Ptr
Strings
	db	low String0, high String0
	db	upper String0, 0
	db	low String1, high String1
	db	upper String1, 0
	db	low String2, high String2
	db	upper String2, 0

; Language code string descriptor
String0
	retlw   (String1-String0)/2	; Size of this descriptor in bytes
	retlw   USB_DESCRIPTOR_STRING	; STRING descriptor type
	retlw   0x09		; Language ID as defined by MS - 0x0409 (low byte)
	retlw   0x04		; (high byte)

; Manufacturer string descriptor
String1
	retlw   (String2-String1)/2	; Size of this descriptor in bytes
	retlw   USB_DESCRIPTOR_STRING	; STRING descriptor type
	retlw   'S'
	retlw   0x00
	retlw   'C'
	retlw   0x00
	retlw   'U'
	retlw   0x00
	retlw   'M'
	retlw   0x00
	retlw   'A'
	retlw   0x00
	retlw   'R'
	retlw   0x00
	retlw   'I'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'T'
	retlw   0x00
	retlw   'E'
	retlw   0x00
	retlw   'C'
	retlw   0x00
	retlw   'H'
	retlw   0x00
	retlw   'N'
	retlw   0x00
	retlw   'O'
	retlw   0x00
	retlw   'L'
	retlw   0x00
	retlw   'O'
	retlw   0x00
	retlw   'G'
	retlw   0x00
	retlw   'I'
	retlw   0x00
	retlw   'E'
	retlw   0x00
	retlw   'S'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   '2'
	retlw   0x00
	retlw   '0'
	retlw   0x00
	retlw   '1'
	retlw   0x00
	retlw   '7'
	retlw   0x00

; Product string descriptor
String2
	retlw   (EndStringDescriptors-String2)/2	; Size of this descriptor in bytes
	retlw   USB_DESCRIPTOR_STRING	; STRING descriptor type
	retlw   'J'
	retlw   0x00
	retlw   'O'
	retlw   0x00
	retlw   'Y'
	retlw   0x00
	retlw   'S'
	retlw   0x00
	retlw   'T'
    retlw   0x00
	retlw   'I'
	retlw   0x00
	retlw   'C'
	retlw   0x00
	retlw   'K'
	retlw   0x00
	retlw   ' '
	retlw   0x00
	retlw   'F'
	retlw   0x00
	retlw   'Y'
	retlw   0x00
EndStringDescriptors

	Endasm

hid_desc_end			' End of skipped over USB assembler code
